# MIT License
#
# Copyright (c) 2018 Evgeny Medvedev, evge.medvedev@gmail.com
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


from datetime import datetime, timezone

from defi_services.utils.graph_operations import GraphOperations, OutOfBoundsError, Point


class EthService(object):
    def __init__(self, web3):
        graph = BlockTimestampGraph(web3)
        self._graph_operations = GraphOperations(graph)

    def get_block_for_timestamp(self, timestamp):
        try:
            block_bounds = self._graph_operations.get_bounds_for_y_coordinate(timestamp)
        except OutOfBoundsError:
            return 'latest'
        return block_bounds[0]

    def get_block_range_for_date(self, date):
        start_datetime = datetime.combine(date, datetime.min.time().replace(tzinfo=timezone.utc))
        end_datetime = datetime.combine(date, datetime.max.time().replace(tzinfo=timezone.utc))
        return self.get_block_range_for_timestamps(start_datetime.timestamp(), end_datetime.timestamp())

    def get_block_range_for_timestamps(self, start_timestamp, end_timestamp):
        start_timestamp = int(start_timestamp)
        end_timestamp = int(end_timestamp)
        if start_timestamp > end_timestamp:
            raise ValueError('start_timestamp must be greater or equal to end_timestamp')

        try:
            start_block_bounds = self._graph_operations.get_bounds_for_y_coordinate(start_timestamp)
        except OutOfBoundsError:
            start_block_bounds = (0, 0)

        try:
            end_block_bounds = self._graph_operations.get_bounds_for_y_coordinate(end_timestamp)
        except OutOfBoundsError as e:
            raise OutOfBoundsError('The existing blocks do not completely cover the given time range') from e

        if start_block_bounds == end_block_bounds and start_block_bounds[0] != start_block_bounds[1]:
            raise ValueError('The given timestamp range does not cover any blocks')

        start_block = start_block_bounds[1]
        end_block = end_block_bounds[0]

        # The genesis block has timestamp 0 but we include it with the 1st block.
        if start_block == 1:
            start_block = 0

        return start_block, end_block


class BlockTimestampGraph(object):
    def __init__(self, web3):
        self._web3 = web3

    def get_first_point(self):
        # Ignore the genesis block as its timestamp is 0
        return block_to_point(self._web3.eth.get_block(1))

    def get_last_point(self):
        return block_to_point(self._web3.eth.get_block('latest'))

    def get_point(self, x):
        return block_to_point(self._web3.eth.get_block(x))


def block_to_point(block):
    return Point(block.number, block.timestamp)



from defi_services.abis.dex.sushiswap.factory_v3_abi import SUSHISWAP_V3_ETH_FACTORY_ABI
from defi_services.abis.dex.uniswap.factory_v2_abi import UNISWAP_FACTORY_ABI
from defi_services.abis.dex.uniswap.nft_token_manager_abi import UNISWAP_V3_NFT_TOKEN_MANGAGER_ABI
from defi_services.abis.dex.uniswap.pool_v3_abi import UNISWAP_V3_POOL_ABI

SUSHISWAP_V0_ETH_INFO = {
    'master_chef_address': '0xc2edad668740f1aa35e4d8f227fb8e17dca888cd',
    'reward_token': '0x6b3595068778dd592e39a122f4f5a5cf09c90fe2',
    'forked': 'uniswap-v2'

}

SUSHISWAP_V2_ETH_INFO = {
    'factory_address': '0xc0aee478e3658e2610c5f7a4a2e1777ce9e4f2ac',
    'master_chef_address': '0xef0881ec094552b2e128cf945ef17a6752b4ec5d',
    'reward_token': '0x6b3595068778dd592e39a122f4f5a5cf09c90fe2',
    'forked': 'uniswap-v2'

}

SUSHISWAP_V2_BSC_INFO = {
    'factory_address': '0xc35dadb65012ec5796536bd9864ed8773abc74c4',
    'master_chef_address': '0x5219c5e32b9fff87f29d5a833832c29134464aaa',
    'reward_token': '0x986cdf0fd180b40c4d6aeaa01ab740b996d8b782',
    'forked': 'uniswap-v2'

}

SUSHISWAP_V2_FANTOM_INFO = {
    'factory_address': '0xc35dadb65012ec5796536bd9864ed8773abc74c4',
    'master_chef_address': '0xf731202a3cf7efa9368c2d7bd613926f7a144db5',
    'reward_token': '0xae75a438b2e0cb8bb01ec1e1e376de11d44477cc',
    'forked': 'uniswap-v2'

}

SUSHISWAP_V2_POLYGON_INFO = {
    'factory_address': '0xc35dadb65012ec5796536bd9864ed8773abc74c4',
    'master_chef_address': '0x0769fd68dfb93167989c6f7254cd0d766fb2841f',
    'reward_token': '0x0b3f868e0be5597d5db7feb59e1cadbb0fdda50a',
    'forked': 'uniswap-v2'

}


SUSHISWAP_V2_AVALANCHE_INFO = {
    'factory_address': '0xc35dadb65012ec5796536bd9864ed8773abc74c4',
    'master_chef_address': '0xe11252176cedd4a493aec9767192c06a04a6b04f',
    'reward_token': '0x37b608519f91f70f2eeb0e5ed9af4061722e4f76',
    'forked': 'uniswap-v2'

}

SUSHISWAP_V2_ARBITRUM_INFO = {
    'factory_address': '0xc35dadb65012ec5796536bd9864ed8773abc74c4',
    'master_chef_address': '0xf4d73326c13a4fc5fd7a064217e12780e9bd62c3',
    'reward_token': '0xd4d42f0b6def4ce0383636770ef773390d85c61a',
    'forked': 'uniswap-v2'

}

SUSHISWAP_V3_ETH_INFO = {
    'factory_address': '0xbaceb8ec6b9355dfc0269c18bac9d6e2bdc29c4f',
    'factory_abi': SUSHISWAP_V3_ETH_FACTORY_ABI,
    'NFT_manager_address': '0x2214a42d8e2a1d20635c2cb0664422c528b6a432',
    'NFT_manager_abi': UNISWAP_V3_NFT_TOKEN_MANGAGER_ABI,
    'pool_abi': UNISWAP_V3_POOL_ABI,
    'forked': 'uniswap-v3'

}
SUSHISWAP_V3_BSC_INFO = {
    'factory_address': '0xbaceb8ec6b9355dfc0269c18bac9d6e2bdc29c4f',
    'factory_abi': SUSHISWAP_V3_ETH_FACTORY_ABI,
    'NFT_manager_address': '0x2214a42d8e2a1d20635c2cb0664422c528b6a432',
    'NFT_manager_abi': UNISWAP_V3_NFT_TOKEN_MANGAGER_ABI,
    'pool_abi': UNISWAP_V3_POOL_ABI,
    'forked': 'uniswap-v3'

}
SUSHISWAP_V3_FANTOM_INFO = {
    'factory_address': '0xbaceb8ec6b9355dfc0269c18bac9d6e2bdc29c4f',
    'factory_abi': SUSHISWAP_V3_ETH_FACTORY_ABI,
    'NFT_manager_address': '0x2214a42d8e2a1d20635c2cb0664422c528b6a432',
    'NFT_manager_abi': UNISWAP_V3_NFT_TOKEN_MANGAGER_ABI,
    'pool_abi': UNISWAP_V3_POOL_ABI,
    'forked': 'uniswap-v3'

}
SUSHISWAP_V3_POLYGON_INFO = {
    'factory_address': '0xbaceb8ec6b9355dfc0269c18bac9d6e2bdc29c4f',
    'factory_abi': SUSHISWAP_V3_ETH_FACTORY_ABI,
    'NFT_manager_address': '0x2214a42d8e2a1d20635c2cb0664422c528b6a432',
    'NFT_manager_abi': UNISWAP_V3_NFT_TOKEN_MANGAGER_ABI,
    'pool_abi': UNISWAP_V3_POOL_ABI,
    'forked': 'uniswap-v3'

}

SUSHISWAP_V3_AVALANCHE_INFO = {
    'factory_address': '0xbaceb8ec6b9355dfc0269c18bac9d6e2bdc29c4f',
    'factory_abi': SUSHISWAP_V3_ETH_FACTORY_ABI,
    'NFT_manager_address': '0x2214a42d8e2a1d20635c2cb0664422c528b6a432',
    'NFT_manager_abi': UNISWAP_V3_NFT_TOKEN_MANGAGER_ABI,
    'pool_abi': UNISWAP_V3_POOL_ABI,
    'forked': 'uniswap-v3'

}

SUSHISWAP_V3_ARBITRUM_INFO = {
    'factory_address': '0xbaceb8ec6b9355dfc0269c18bac9d6e2bdc29c4f',
    'factory_abi': SUSHISWAP_V3_ETH_FACTORY_ABI,
    'NFT_manager_address': '0x2214a42d8e2a1d20635c2cb0664422c528b6a432',
    'NFT_manager_abi': UNISWAP_V3_NFT_TOKEN_MANGAGER_ABI,
    'pool_abi': UNISWAP_V3_POOL_ABI,
    'forked': 'uniswap-v3'

}



import json

LIDO_STAKING_ROUTER_ABI = json.loads('''
[
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_depositContract",
        "type": "address"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "constructor"
  },
  {
    "inputs": [],
    "name": "AppAuthLidoFailed",
    "type": "error"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "firstArrayLength",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "secondArrayLength",
        "type": "uint256"
      }
    ],
    "name": "ArraysLengthMismatch",
    "type": "error"
  },
  {
    "inputs": [],
    "name": "DepositContractZeroAddress",
    "type": "error"
  },
  {
    "inputs": [],
    "name": "DirectETHTransfer",
    "type": "error"
  },
  {
    "inputs": [],
    "name": "EmptyWithdrawalsCredentials",
    "type": "error"
  },
  {
    "inputs": [],
    "name": "ExitedValidatorsCountCannotDecrease",
    "type": "error"
  },
  {
    "inputs": [],
    "name": "InvalidContractVersionIncrement",
    "type": "error"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "etherValue",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "depositsCount",
        "type": "uint256"
      }
    ],
    "name": "InvalidDepositsValue",
    "type": "error"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "actual",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "expected",
        "type": "uint256"
      }
    ],
    "name": "InvalidPublicKeysBatchLength",
    "type": "error"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "code",
        "type": "uint256"
      }
    ],
    "name": "InvalidReportData",
    "type": "error"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "actual",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "expected",
        "type": "uint256"
      }
    ],
    "name": "InvalidSignaturesBatchLength",
    "type": "error"
  },
  {
    "inputs": [],
    "name": "NonZeroContractVersionOnInit",
    "type": "error"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "reportedExitedValidatorsCount",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "depositedValidatorsCount",
        "type": "uint256"
      }
    ],
    "name": "ReportedExitedValidatorsExceedDeposited",
    "type": "error"
  },
  {
    "inputs": [],
    "name": "StakingModuleAddressExists",
    "type": "error"
  },
  {
    "inputs": [],
    "name": "StakingModuleNotActive",
    "type": "error"
  },
  {
    "inputs": [],
    "name": "StakingModuleNotPaused",
    "type": "error"
  },
  {
    "inputs": [],
    "name": "StakingModuleStatusTheSame",
    "type": "error"
  },
  {
    "inputs": [],
    "name": "StakingModuleUnregistered",
    "type": "error"
  },
  {
    "inputs": [],
    "name": "StakingModuleWrongName",
    "type": "error"
  },
  {
    "inputs": [],
    "name": "StakingModulesLimitExceeded",
    "type": "error"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "expected",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "received",
        "type": "uint256"
      }
    ],
    "name": "UnexpectedContractVersion",
    "type": "error"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "currentModuleExitedValidatorsCount",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "currentNodeOpExitedValidatorsCount",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "currentNodeOpStuckValidatorsCount",
        "type": "uint256"
      }
    ],
    "name": "UnexpectedCurrentValidatorsCount",
    "type": "error"
  },
  {
    "inputs": [],
    "name": "UnrecoverableModuleError",
    "type": "error"
  },
  {
    "inputs": [
      {
        "internalType": "string",
        "name": "field",
        "type": "string"
      }
    ],
    "name": "ValueOver100Percent",
    "type": "error"
  },
  {
    "inputs": [
      {
        "internalType": "string",
        "name": "field",
        "type": "string"
      }
    ],
    "name": "ZeroAddress",
    "type": "error"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "version",
        "type": "uint256"
      }
    ],
    "name": "ContractVersionSet",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "uint256",
        "name": "stakingModuleId",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "bytes",
        "name": "lowLevelRevertData",
        "type": "bytes"
      }
    ],
    "name": "ExitedAndStuckValidatorsCountsUpdateFailed",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "uint256",
        "name": "stakingModuleId",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "bytes",
        "name": "lowLevelRevertData",
        "type": "bytes"
      }
    ],
    "name": "RewardsMintedReportFailed",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "indexed": true,
        "internalType": "bytes32",
        "name": "previousAdminRole",
        "type": "bytes32"
      },
      {
        "indexed": true,
        "internalType": "bytes32",
        "name": "newAdminRole",
        "type": "bytes32"
      }
    ],
    "name": "RoleAdminChanged",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "account",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "sender",
        "type": "address"
      }
    ],
    "name": "RoleGranted",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "account",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "sender",
        "type": "address"
      }
    ],
    "name": "RoleRevoked",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "uint256",
        "name": "stakingModuleId",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "address",
        "name": "stakingModule",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "string",
        "name": "name",
        "type": "string"
      },
      {
        "indexed": false,
        "internalType": "address",
        "name": "createdBy",
        "type": "address"
      }
    ],
    "name": "StakingModuleAdded",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "uint256",
        "name": "stakingModuleId",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "unreportedExitedValidatorsCount",
        "type": "uint256"
      }
    ],
    "name": "StakingModuleExitedValidatorsIncompleteReporting",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "uint256",
        "name": "stakingModuleId",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "stakingModuleFee",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "treasuryFee",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "address",
        "name": "setBy",
        "type": "address"
      }
    ],
    "name": "StakingModuleFeesSet",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "uint256",
        "name": "stakingModuleId",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "enum StakingRouter.StakingModuleStatus",
        "name": "status",
        "type": "uint8"
      },
      {
        "indexed": false,
        "internalType": "address",
        "name": "setBy",
        "type": "address"
      }
    ],
    "name": "StakingModuleStatusSet",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "uint256",
        "name": "stakingModuleId",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "targetShare",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "address",
        "name": "setBy",
        "type": "address"
      }
    ],
    "name": "StakingModuleTargetShareSet",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "uint256",
        "name": "stakingModuleId",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "amount",
        "type": "uint256"
      }
    ],
    "name": "StakingRouterETHDeposited",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "bytes32",
        "name": "withdrawalCredentials",
        "type": "bytes32"
      },
      {
        "indexed": false,
        "internalType": "address",
        "name": "setBy",
        "type": "address"
      }
    ],
    "name": "WithdrawalCredentialsSet",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "uint256",
        "name": "stakingModuleId",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "bytes",
        "name": "lowLevelRevertData",
        "type": "bytes"
      }
    ],
    "name": "WithdrawalsCredentialsChangeFailed",
    "type": "event"
  },
  {
    "inputs": [],
    "name": "DEFAULT_ADMIN_ROLE",
    "outputs": [
      {
        "internalType": "bytes32",
        "name": "",
        "type": "bytes32"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "DEPOSIT_CONTRACT",
    "outputs": [
      {
        "internalType": "contract IDepositContract",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "FEE_PRECISION_POINTS",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "MANAGE_WITHDRAWAL_CREDENTIALS_ROLE",
    "outputs": [
      {
        "internalType": "bytes32",
        "name": "",
        "type": "bytes32"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "MAX_STAKING_MODULES_COUNT",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "MAX_STAKING_MODULE_NAME_LENGTH",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "REPORT_EXITED_VALIDATORS_ROLE",
    "outputs": [
      {
        "internalType": "bytes32",
        "name": "",
        "type": "bytes32"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "REPORT_REWARDS_MINTED_ROLE",
    "outputs": [
      {
        "internalType": "bytes32",
        "name": "",
        "type": "bytes32"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "STAKING_MODULE_MANAGE_ROLE",
    "outputs": [
      {
        "internalType": "bytes32",
        "name": "",
        "type": "bytes32"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "STAKING_MODULE_PAUSE_ROLE",
    "outputs": [
      {
        "internalType": "bytes32",
        "name": "",
        "type": "bytes32"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "STAKING_MODULE_RESUME_ROLE",
    "outputs": [
      {
        "internalType": "bytes32",
        "name": "",
        "type": "bytes32"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "TOTAL_BASIS_POINTS",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "UNSAFE_SET_EXITED_VALIDATORS_ROLE",
    "outputs": [
      {
        "internalType": "bytes32",
        "name": "",
        "type": "bytes32"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "string",
        "name": "_name",
        "type": "string"
      },
      {
        "internalType": "address",
        "name": "_stakingModuleAddress",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_targetShare",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "_stakingModuleFee",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "_treasuryFee",
        "type": "uint256"
      }
    ],
    "name": "addStakingModule",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_depositsCount",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      },
      {
        "internalType": "bytes",
        "name": "_depositCalldata",
        "type": "bytes"
      }
    ],
    "name": "deposit",
    "outputs": [],
    "stateMutability": "payable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      }
    ],
    "name": "getAllNodeOperatorDigests",
    "outputs": [
      {
        "components": [
          {
            "internalType": "uint256",
            "name": "id",
            "type": "uint256"
          },
          {
            "internalType": "bool",
            "name": "isActive",
            "type": "bool"
          },
          {
            "components": [
              {
                "internalType": "bool",
                "name": "isTargetLimitActive",
                "type": "bool"
              },
              {
                "internalType": "uint256",
                "name": "targetValidatorsCount",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "stuckValidatorsCount",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "refundedValidatorsCount",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "stuckPenaltyEndTimestamp",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "totalExitedValidators",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "totalDepositedValidators",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "depositableValidatorsCount",
                "type": "uint256"
              }
            ],
            "internalType": "struct StakingRouter.NodeOperatorSummary",
            "name": "summary",
            "type": "tuple"
          }
        ],
        "internalType": "struct StakingRouter.NodeOperatorDigest[]",
        "name": "",
        "type": "tuple[]"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "getAllStakingModuleDigests",
    "outputs": [
      {
        "components": [
          {
            "internalType": "uint256",
            "name": "nodeOperatorsCount",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "activeNodeOperatorsCount",
            "type": "uint256"
          },
          {
            "components": [
              {
                "internalType": "uint24",
                "name": "id",
                "type": "uint24"
              },
              {
                "internalType": "address",
                "name": "stakingModuleAddress",
                "type": "address"
              },
              {
                "internalType": "uint16",
                "name": "stakingModuleFee",
                "type": "uint16"
              },
              {
                "internalType": "uint16",
                "name": "treasuryFee",
                "type": "uint16"
              },
              {
                "internalType": "uint16",
                "name": "targetShare",
                "type": "uint16"
              },
              {
                "internalType": "uint8",
                "name": "status",
                "type": "uint8"
              },
              {
                "internalType": "string",
                "name": "name",
                "type": "string"
              },
              {
                "internalType": "uint64",
                "name": "lastDepositAt",
                "type": "uint64"
              },
              {
                "internalType": "uint256",
                "name": "lastDepositBlock",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "exitedValidatorsCount",
                "type": "uint256"
              }
            ],
            "internalType": "struct StakingRouter.StakingModule",
            "name": "state",
            "type": "tuple"
          },
          {
            "components": [
              {
                "internalType": "uint256",
                "name": "totalExitedValidators",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "totalDepositedValidators",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "depositableValidatorsCount",
                "type": "uint256"
              }
            ],
            "internalType": "struct StakingRouter.StakingModuleSummary",
            "name": "summary",
            "type": "tuple"
          }
        ],
        "internalType": "struct StakingRouter.StakingModuleDigest[]",
        "name": "",
        "type": "tuple[]"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "getContractVersion",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_depositsCount",
        "type": "uint256"
      }
    ],
    "name": "getDepositsAllocation",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "allocated",
        "type": "uint256"
      },
      {
        "internalType": "uint256[]",
        "name": "allocations",
        "type": "uint256[]"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "getLido",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      },
      {
        "internalType": "uint256[]",
        "name": "_nodeOperatorIds",
        "type": "uint256[]"
      }
    ],
    "name": "getNodeOperatorDigests",
    "outputs": [
      {
        "components": [
          {
            "internalType": "uint256",
            "name": "id",
            "type": "uint256"
          },
          {
            "internalType": "bool",
            "name": "isActive",
            "type": "bool"
          },
          {
            "components": [
              {
                "internalType": "bool",
                "name": "isTargetLimitActive",
                "type": "bool"
              },
              {
                "internalType": "uint256",
                "name": "targetValidatorsCount",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "stuckValidatorsCount",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "refundedValidatorsCount",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "stuckPenaltyEndTimestamp",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "totalExitedValidators",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "totalDepositedValidators",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "depositableValidatorsCount",
                "type": "uint256"
              }
            ],
            "internalType": "struct StakingRouter.NodeOperatorSummary",
            "name": "summary",
            "type": "tuple"
          }
        ],
        "internalType": "struct StakingRouter.NodeOperatorDigest[]",
        "name": "digests",
        "type": "tuple[]"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "_offset",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "_limit",
        "type": "uint256"
      }
    ],
    "name": "getNodeOperatorDigests",
    "outputs": [
      {
        "components": [
          {
            "internalType": "uint256",
            "name": "id",
            "type": "uint256"
          },
          {
            "internalType": "bool",
            "name": "isActive",
            "type": "bool"
          },
          {
            "components": [
              {
                "internalType": "bool",
                "name": "isTargetLimitActive",
                "type": "bool"
              },
              {
                "internalType": "uint256",
                "name": "targetValidatorsCount",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "stuckValidatorsCount",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "refundedValidatorsCount",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "stuckPenaltyEndTimestamp",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "totalExitedValidators",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "totalDepositedValidators",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "depositableValidatorsCount",
                "type": "uint256"
              }
            ],
            "internalType": "struct StakingRouter.NodeOperatorSummary",
            "name": "summary",
            "type": "tuple"
          }
        ],
        "internalType": "struct StakingRouter.NodeOperatorDigest[]",
        "name": "",
        "type": "tuple[]"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "_nodeOperatorId",
        "type": "uint256"
      }
    ],
    "name": "getNodeOperatorSummary",
    "outputs": [
      {
        "components": [
          {
            "internalType": "bool",
            "name": "isTargetLimitActive",
            "type": "bool"
          },
          {
            "internalType": "uint256",
            "name": "targetValidatorsCount",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "stuckValidatorsCount",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "refundedValidatorsCount",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "stuckPenaltyEndTimestamp",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "totalExitedValidators",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "totalDepositedValidators",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "depositableValidatorsCount",
            "type": "uint256"
          }
        ],
        "internalType": "struct StakingRouter.NodeOperatorSummary",
        "name": "summary",
        "type": "tuple"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      }
    ],
    "name": "getRoleAdmin",
    "outputs": [
      {
        "internalType": "bytes32",
        "name": "",
        "type": "bytes32"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "internalType": "uint256",
        "name": "index",
        "type": "uint256"
      }
    ],
    "name": "getRoleMember",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      }
    ],
    "name": "getRoleMemberCount",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "getStakingFeeAggregateDistribution",
    "outputs": [
      {
        "internalType": "uint96",
        "name": "modulesFee",
        "type": "uint96"
      },
      {
        "internalType": "uint96",
        "name": "treasuryFee",
        "type": "uint96"
      },
      {
        "internalType": "uint256",
        "name": "basePrecision",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "getStakingFeeAggregateDistributionE4Precision",
    "outputs": [
      {
        "internalType": "uint16",
        "name": "modulesFee",
        "type": "uint16"
      },
      {
        "internalType": "uint16",
        "name": "treasuryFee",
        "type": "uint16"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      }
    ],
    "name": "getStakingModule",
    "outputs": [
      {
        "components": [
          {
            "internalType": "uint24",
            "name": "id",
            "type": "uint24"
          },
          {
            "internalType": "address",
            "name": "stakingModuleAddress",
            "type": "address"
          },
          {
            "internalType": "uint16",
            "name": "stakingModuleFee",
            "type": "uint16"
          },
          {
            "internalType": "uint16",
            "name": "treasuryFee",
            "type": "uint16"
          },
          {
            "internalType": "uint16",
            "name": "targetShare",
            "type": "uint16"
          },
          {
            "internalType": "uint8",
            "name": "status",
            "type": "uint8"
          },
          {
            "internalType": "string",
            "name": "name",
            "type": "string"
          },
          {
            "internalType": "uint64",
            "name": "lastDepositAt",
            "type": "uint64"
          },
          {
            "internalType": "uint256",
            "name": "lastDepositBlock",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "exitedValidatorsCount",
            "type": "uint256"
          }
        ],
        "internalType": "struct StakingRouter.StakingModule",
        "name": "",
        "type": "tuple"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      }
    ],
    "name": "getStakingModuleActiveValidatorsCount",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "activeValidatorsCount",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256[]",
        "name": "_stakingModuleIds",
        "type": "uint256[]"
      }
    ],
    "name": "getStakingModuleDigests",
    "outputs": [
      {
        "components": [
          {
            "internalType": "uint256",
            "name": "nodeOperatorsCount",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "activeNodeOperatorsCount",
            "type": "uint256"
          },
          {
            "components": [
              {
                "internalType": "uint24",
                "name": "id",
                "type": "uint24"
              },
              {
                "internalType": "address",
                "name": "stakingModuleAddress",
                "type": "address"
              },
              {
                "internalType": "uint16",
                "name": "stakingModuleFee",
                "type": "uint16"
              },
              {
                "internalType": "uint16",
                "name": "treasuryFee",
                "type": "uint16"
              },
              {
                "internalType": "uint16",
                "name": "targetShare",
                "type": "uint16"
              },
              {
                "internalType": "uint8",
                "name": "status",
                "type": "uint8"
              },
              {
                "internalType": "string",
                "name": "name",
                "type": "string"
              },
              {
                "internalType": "uint64",
                "name": "lastDepositAt",
                "type": "uint64"
              },
              {
                "internalType": "uint256",
                "name": "lastDepositBlock",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "exitedValidatorsCount",
                "type": "uint256"
              }
            ],
            "internalType": "struct StakingRouter.StakingModule",
            "name": "state",
            "type": "tuple"
          },
          {
            "components": [
              {
                "internalType": "uint256",
                "name": "totalExitedValidators",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "totalDepositedValidators",
                "type": "uint256"
              },
              {
                "internalType": "uint256",
                "name": "depositableValidatorsCount",
                "type": "uint256"
              }
            ],
            "internalType": "struct StakingRouter.StakingModuleSummary",
            "name": "summary",
            "type": "tuple"
          }
        ],
        "internalType": "struct StakingRouter.StakingModuleDigest[]",
        "name": "digests",
        "type": "tuple[]"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "getStakingModuleIds",
    "outputs": [
      {
        "internalType": "uint256[]",
        "name": "stakingModuleIds",
        "type": "uint256[]"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      }
    ],
    "name": "getStakingModuleIsActive",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      }
    ],
    "name": "getStakingModuleIsDepositsPaused",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      }
    ],
    "name": "getStakingModuleIsStopped",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      }
    ],
    "name": "getStakingModuleLastDepositBlock",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "_maxDepositsValue",
        "type": "uint256"
      }
    ],
    "name": "getStakingModuleMaxDepositsCount",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      }
    ],
    "name": "getStakingModuleNonce",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      }
    ],
    "name": "getStakingModuleStatus",
    "outputs": [
      {
        "internalType": "enum StakingRouter.StakingModuleStatus",
        "name": "",
        "type": "uint8"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      }
    ],
    "name": "getStakingModuleSummary",
    "outputs": [
      {
        "components": [
          {
            "internalType": "uint256",
            "name": "totalExitedValidators",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "totalDepositedValidators",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "depositableValidatorsCount",
            "type": "uint256"
          }
        ],
        "internalType": "struct StakingRouter.StakingModuleSummary",
        "name": "summary",
        "type": "tuple"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "getStakingModules",
    "outputs": [
      {
        "components": [
          {
            "internalType": "uint24",
            "name": "id",
            "type": "uint24"
          },
          {
            "internalType": "address",
            "name": "stakingModuleAddress",
            "type": "address"
          },
          {
            "internalType": "uint16",
            "name": "stakingModuleFee",
            "type": "uint16"
          },
          {
            "internalType": "uint16",
            "name": "treasuryFee",
            "type": "uint16"
          },
          {
            "internalType": "uint16",
            "name": "targetShare",
            "type": "uint16"
          },
          {
            "internalType": "uint8",
            "name": "status",
            "type": "uint8"
          },
          {
            "internalType": "string",
            "name": "name",
            "type": "string"
          },
          {
            "internalType": "uint64",
            "name": "lastDepositAt",
            "type": "uint64"
          },
          {
            "internalType": "uint256",
            "name": "lastDepositBlock",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "exitedValidatorsCount",
            "type": "uint256"
          }
        ],
        "internalType": "struct StakingRouter.StakingModule[]",
        "name": "res",
        "type": "tuple[]"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "getStakingModulesCount",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "getStakingRewardsDistribution",
    "outputs": [
      {
        "internalType": "address[]",
        "name": "recipients",
        "type": "address[]"
      },
      {
        "internalType": "uint256[]",
        "name": "stakingModuleIds",
        "type": "uint256[]"
      },
      {
        "internalType": "uint96[]",
        "name": "stakingModuleFees",
        "type": "uint96[]"
      },
      {
        "internalType": "uint96",
        "name": "totalFee",
        "type": "uint96"
      },
      {
        "internalType": "uint256",
        "name": "precisionPoints",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "getTotalFeeE4Precision",
    "outputs": [
      {
        "internalType": "uint16",
        "name": "totalFee",
        "type": "uint16"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "getWithdrawalCredentials",
    "outputs": [
      {
        "internalType": "bytes32",
        "name": "",
        "type": "bytes32"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "grantRole",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "hasRole",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      }
    ],
    "name": "hasStakingModule",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_admin",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_lido",
        "type": "address"
      },
      {
        "internalType": "bytes32",
        "name": "_withdrawalCredentials",
        "type": "bytes32"
      }
    ],
    "name": "initialize",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "onValidatorsCountsByNodeOperatorReportingFinished",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      }
    ],
    "name": "pauseStakingModule",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "renounceRole",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256[]",
        "name": "_stakingModuleIds",
        "type": "uint256[]"
      },
      {
        "internalType": "uint256[]",
        "name": "_totalShares",
        "type": "uint256[]"
      }
    ],
    "name": "reportRewardsMinted",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      },
      {
        "internalType": "bytes",
        "name": "_nodeOperatorIds",
        "type": "bytes"
      },
      {
        "internalType": "bytes",
        "name": "_exitedValidatorsCounts",
        "type": "bytes"
      }
    ],
    "name": "reportStakingModuleExitedValidatorsCountByNodeOperator",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      },
      {
        "internalType": "bytes",
        "name": "_nodeOperatorIds",
        "type": "bytes"
      },
      {
        "internalType": "bytes",
        "name": "_stuckValidatorsCounts",
        "type": "bytes"
      }
    ],
    "name": "reportStakingModuleStuckValidatorsCountByNodeOperator",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      }
    ],
    "name": "resumeStakingModule",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "revokeRole",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      },
      {
        "internalType": "enum StakingRouter.StakingModuleStatus",
        "name": "_status",
        "type": "uint8"
      }
    ],
    "name": "setStakingModuleStatus",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "_withdrawalCredentials",
        "type": "bytes32"
      }
    ],
    "name": "setWithdrawalCredentials",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes4",
        "name": "interfaceId",
        "type": "bytes4"
      }
    ],
    "name": "supportsInterface",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "_nodeOperatorId",
        "type": "uint256"
      },
      {
        "internalType": "bool",
        "name": "_triggerUpdateFinish",
        "type": "bool"
      },
      {
        "components": [
          {
            "internalType": "uint256",
            "name": "currentModuleExitedValidatorsCount",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "currentNodeOperatorExitedValidatorsCount",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "currentNodeOperatorStuckValidatorsCount",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "newModuleExitedValidatorsCount",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "newNodeOperatorExitedValidatorsCount",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "newNodeOperatorStuckValidatorsCount",
            "type": "uint256"
          }
        ],
        "internalType": "struct StakingRouter.ValidatorsCountsCorrection",
        "name": "_correction",
        "type": "tuple"
      }
    ],
    "name": "unsafeSetExitedValidatorsCount",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256[]",
        "name": "_stakingModuleIds",
        "type": "uint256[]"
      },
      {
        "internalType": "uint256[]",
        "name": "_exitedValidatorsCounts",
        "type": "uint256[]"
      }
    ],
    "name": "updateExitedValidatorsCountByStakingModule",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "_nodeOperatorId",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "_refundedValidatorsCount",
        "type": "uint256"
      }
    ],
    "name": "updateRefundedValidatorsCount",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "_targetShare",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "_stakingModuleFee",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "_treasuryFee",
        "type": "uint256"
      }
    ],
    "name": "updateStakingModule",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_stakingModuleId",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "_nodeOperatorId",
        "type": "uint256"
      },
      {
        "internalType": "bool",
        "name": "_isTargetLimitActive",
        "type": "bool"
      },
      {
        "internalType": "uint256",
        "name": "_targetLimit",
        "type": "uint256"
      }
    ],
    "name": "updateTargetValidatorsLimits",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "stateMutability": "payable",
    "type": "receive"
  }
]
''')


class Denoms:
    cosmos = {
        "uatom": {
            "name": "ATOM",
            "denom": "uatom",
            "decimal": 6
        },
        "ibc/0025f8a87464a471e66b234c4f93aec5b4da3d42d7986451a059273426290dd5": {
            "name": "NTRN",
            "denom": "ibc/0025f8a87464a471e66b234c4f93aec5b4da3d42d7986451a059273426290dd5",
            "decimal": 6
        },
        "ibc/f663521bf1836b00f5f177680f74bfb9a8b5654a694d0d2bc249e03cf2509013": {
            "name": "USDC",
            "denom": "ibc/f663521bf1836b00f5f177680f74bfb9a8b5654a694d0d2bc249e03cf2509013",
            "decimal": 6
        },
        "ibc/b05539b66b72e2739b986b86391e5d08f12b8d5d2c2a7f8f8cf9adf674dfa231": {
            "name": "stATOM",
            "denom": "ibc/b05539b66b72e2739b986b86391e5d08f12b8d5d2c2a7f8f8cf9adf674dfa231",
            "decimal": 6
        },
        "ibc/2181aab0218eac24bc9f86bd1364fbbfa3e6e3fcc25e88e3e68c15dc6e752d86": {
            "name": "AKT",
            "denom": "ibc/2181aab0218eac24bc9f86bd1364fbbfa3e6e3fcc25e88e3e68c15dc6e752d86",
            "decimal": 6
        },
        "ibc/6b8a3f5c2ad51cd6171fa41a7e8c35ad594ab69226438db94450436ea57b3a89": {
            "name": "STRD",
            "denom": "ibc/6b8a3f5c2ad51cd6171fa41a7e8c35ad594ab69226438db94450436ea57b3a89",
            "decimal": 6
        },
        "ibc/14f9bc3e44b8a9c1be1fb08980fab87034c9905ef17cf2f5008fc085218811cc": {
            "name": "OSMO",
            "denom": "ibc/14f9bc3e44b8a9c1be1fb08980fab87034c9905ef17cf2f5008fc085218811cc",
            "decimal": 6
        },
        "ibc/e7d5e9d0e9bf8b7354929a817dd28d4d017e745f638954764aa88522a7a409ec": {
            "name": "BTSG",
            "denom": "ibc/e7d5e9d0e9bf8b7354929a817dd28d4d017e745f638954764aa88522a7a409ec",
            "decimal": 6
        },
        "ibc/054892d6bb43af8b93aac28aa5fd7019d2c59a15dafd6f45c1fa2bf9bda22454": {
            "name": "stOSMO",
            "denom": "ibc/054892d6bb43af8b93aac28aa5fd7019d2c59a15dafd6f45c1fa2bf9bda22454",
            "decimal": 6
        },
        "ibc/adbec1a7ac2fef73e06b066a1c94dab6c27924ef7ea3f5a43378150009620284": {
            "name": "BCNA",
            "denom": "ibc/adbec1a7ac2fef73e06b066a1c94dab6c27924ef7ea3f5a43378150009620284",
            "decimal": 6
        },
        "ibc/12da42304ee1ce96071f712aa4d58186ad11c3165c0dcda71e017a54f3935e66": {
            "name": "IRIS",
            "denom": "ibc/12da42304ee1ce96071f712aa4d58186ad11c3165c0dcda71e017a54f3935e66",
            "decimal": 6
        },
        "ibc/b011c1a0ad5e717f674ba59fd8e05b2f946e4fd41c9cb3311c95f7ed4b815620": {
            "name": "stINJ",
            "denom": "ibc/b011c1a0ad5e717f674ba59fd8e05b2f946e4fd41c9cb3311c95f7ed4b815620",
            "decimal": 18
        },
        "ibc/81d08bc39fb520ebd948cf017910dd69702d34bf5ac160f76d3b5cfc444ebce0": {
            "name": "XPRT",
            "denom": "ibc/81d08bc39fb520ebd948cf017910dd69702d34bf5ac160f76d3b5cfc444ebce0",
            "decimal": 6
        },
        "ibc/c932adfe2b4216397a4f17458b6e4468499b86c3bc8116180f85d799d6f5cc1b": {
            "name": "CRO",
            "denom": "ibc/c932adfe2b4216397a4f17458b6e4468499b86c3bc8116180f85d799d6f5cc1b",
            "decimal": 8
        },
        "ibc/42e47a5ba708ebe6e0c227006254f2784e209f4dbd3c6bb77edc4b29ef875e8e": {
            "name": "DVPN",
            "denom": "ibc/42e47a5ba708ebe6e0c227006254f2784e209f4dbd3c6bb77edc4b29ef875e8e",
            "decimal": 6
        },
        "ibc/1fbdd58d438b4d04d26cbfb2e722c18984a0f1a52468c4f42f37d102f3d3f399": {
            "name": "REGEN",
            "denom": "ibc/1fbdd58d438b4d04d26cbfb2e722c18984a0f1a52468c4f42f37d102f3d3f399",
            "decimal": 6
        },
        "ibc/19dd710119533524061885a6f190b18af28d9537e2bae37f32a62c1a25979287": {
            "name": "EVMOS",
            "denom": "ibc/19dd710119533524061885a6f190b18af28d9537e2bae37f32a62c1a25979287",
            "decimal": 18
        },
        "ibc/88dcaa43a9cd099e1f9bbb80b9a90f64782eba115a84b2cd8398757ada4f4b40": {
            "name": "stJUNO",
            "denom": "ibc/88dcaa43a9cd099e1f9bbb80b9a90f64782eba115a84b2cd8398757ada4f4b40",
            "decimal": 6
        },
        "ibc/715bd634cf4d914c3ee93b0f8a9d2514b743f6fe36bc80263d1bc5ee4b3c5d40": {
            "name": "stSTARS",
            "denom": "ibc/715bd634cf4d914c3ee93b0f8a9d2514b743f6fe36bc80263d1bc5ee4b3c5d40",
            "decimal": 6
        },
        "ibc/6469bda6f62c4f4b8f76629fa1e72a02a3d1dd9e2b22ddb3c3b2296dead29ab8": {
            "name": "INJ",
            "denom": "ibc/6469bda6f62c4f4b8f76629fa1e72a02a3d1dd9e2b22ddb3c3b2296dead29ab8",
            "decimal": 18
        },
        "ibc/5bb694d466ccf099ef73f165f88472af51d9c4991eaa42bd1168c5304712cc0d": {
            "name": "ION",
            "denom": "ibc/5bb694d466ccf099ef73f165f88472af51d9c4991eaa42bd1168c5304712cc0d",
            "decimal": 6
        },
        "ibc/533e5ffc606fd11b8dca309c66afd6a1f046ef784a73f323a332cf6823f0ea87": {
            "name": "XKI",
            "denom": "ibc/533e5ffc606fd11b8dca309c66afd6a1f046ef784a73f323a332cf6823f0ea87",
            "decimal": 6
        },
        "ibc/1542f8dc70e7999691e991e1edeb1b47e65e3a217b1649d347098ee48acb580f": {
            "name": "SCRT",
            "denom": "ibc/1542f8dc70e7999691e991e1edeb1b47e65e3a217b1649d347098ee48acb580f",
            "decimal": 6
        },
        "ibc/cdab23da5495290063363bd1c3499e26189036302dc689985a7e23f8df8d8db0": {
            "name": "JUNO",
            "denom": "ibc/cdab23da5495290063363bd1c3499e26189036302dc689985a7e23f8df8d8db0",
            "decimal": 6
        },
        "ibc/68a333688e5b07451f95555f8fe510e43ef9d3d44df0909964f92081ef9be5a7": {
            "name": "IOV",
            "denom": "ibc/68a333688e5b07451f95555f8fe510e43ef9d3d44df0909964f92081ef9be5a7",
            "decimal": 6
        },
        "ibc/1d5826f7ede6e3b13009fef994dc9caaf15cc24ca7a9ff436ffb2e56fd72f54f": {
            "name": "LIKE",
            "denom": "ibc/1d5826f7ede6e3b13009fef994dc9caaf15cc24ca7a9ff436ffb2e56fd72f54f",
            "decimal": 9
        },
        "ibc/dec41a02e47658d40fc71e5a35a9c807111f5a6662a3fb5da84c4e6f53e616b3": {
            "name": "UMEE",
            "denom": "ibc/dec41a02e47658d40fc71e5a35a9c807111f5a6662a3fb5da84c4e6f53e616b3",
            "decimal": 6
        },
        "ibc/f5ed5f3dc6f0ef73fa455337c027fe91abcb375116bf51a228e44c493e020a09": {
            "name": "ROWAN",
            "denom": "ibc/f5ed5f3dc6f0ef73fa455337c027fe91abcb375116bf51a228e44c493e020a09",
            "decimal": 18
        },
        "ibc/5cae744c89bc70ae7b38019a1edf83199b7e10f00f160e7f4f12bca7a32a7ee5": {
            "name": "stLUNA",
            "denom": "ibc/5cae744c89bc70ae7b38019a1edf83199b7e10f00f160e7f4f12bca7a32a7ee5",
            "decimal": 6
        },
        "ibc/8870c4203cebf2279ba065e3de95fc3f8e05a4a93424e7dc707a21514be353a0": {
            "name": "KAVA",
            "denom": "ibc/8870c4203cebf2279ba065e3de95fc3f8e05a4a93424e7dc707a21514be353a0",
            "decimal": 6
        },
        "ibc/b93f321238f7bb15ab5b882660aae72286c8e9035de34e2b30f60e54c623c63c": {
            "name": "EEUR",
            "denom": "ibc/b93f321238f7bb15ab5b882660aae72286c8e9035de34e2b30f60e54c623c63c",
            "decimal": 6
        },
        "ibc/e070ce91cc4bd15aec9b5788c0826755aad35052a3037e9ac62be70b4c9a7dbb": {
            "name": "NGM",
            "denom": "ibc/e070ce91cc4bd15aec9b5788c0826755aad35052a3037e9ac62be70b4c9a7dbb",
            "decimal": 6
        },
        "ibc/835ee9d00c35d72128f195b50f8a89eb83e5011c43ea0aa00d16348e2208febb": {
            "name": "bCRE",
            "denom": "ibc/835ee9d00c35d72128f195b50f8a89eb83e5011c43ea0aa00d16348e2208febb",
            "decimal": 6
        },
        "ibc/3f18d520ce791a40357d061fad657ced6b21d023f229eaf131d7fe7ce6f488bd": {
            "name": "CRE",
            "denom": "ibc/3f18d520ce791a40357d061fad657ced6b21d023f229eaf131d7fe7ce6f488bd",
            "decimal": 6
        },
        "ibc/db9aaadfbe21e014373ef97141d451f453b9b9a2c8b65f5cf8e7ce57531b6850": {
            "name": "CRO",
            "denom": "ibc/db9aaadfbe21e014373ef97141d451f453b9b9a2c8b65f5cf8e7ce57531b6850",
            "decimal": 8
        },
        "ibc/20a7dc8e24709e6f1ee0f4e832c2ed345add77425890482a349ae3c43cac6b2c": {
            "name": "ATOLO",
            "denom": "ibc/20a7dc8e24709e6f1ee0f4e832c2ed345add77425890482a349ae3c43cac6b2c",
            "decimal": 6
        }
    }
    orai = {
        "orai1gjdmce2cfu2ld7cw0p9p7mkmssnry4zhwz3tmsmah435pkdv9qkqg2n7qm": {
            "name": "Aave USDT",
            "denom": "orai1gjdmce2cfu2ld7cw0p9p7mkmssnry4zhwz3tmsmah435pkdv9qkqg2n7qm",
            "decimal": 6
        },
        "orai10ldgzued6zjp0mkqwsv2mux3ml50l97c74x8sg": {
            "name": "aiRight",
            "denom": "orai10ldgzued6zjp0mkqwsv2mux3ml50l97c74x8sg",
            "decimal": 6,
            "base_denom": "AIRI"
        },
        "orai10g6frpysmdgw5tdqke47als6f97aqmr8s3cljsvjce4n5enjftcqtamzsd": {
            "name": "Bitcoin",
            "denom": "orai10g6frpysmdgw5tdqke47als6f97aqmr8s3cljsvjce4n5enjftcqtamzsd",
            "decimal": 6
        },
        "ibc/a2e2eec9057a4a1c2c0a6a4c78b0239118df5f278830f50b4a6bdd7a66506b78": {
            "name": "Cosmos Hub",
            "denom": "ibc/a2e2eec9057a4a1c2c0a6a4c78b0239118df5f278830f50b4a6bdd7a66506b78",
            "decimal": 6
        },
        "orai19rtmkk6sn4tppvjmp5d5zj6gfsdykrl5rw2euu5gwur3luheuuusesqn49": {
            "name": "Injective",
            "denom": "orai19rtmkk6sn4tppvjmp5d5zj6gfsdykrl5rw2euu5gwur3luheuuusesqn49",
            "decimal": 6
        },
        "orai1nd4r053e3kgedgld2ymen8l9yrw8xpjyaal7j5": {
            "name": "Kawaii Islands",
            "denom": "orai1nd4r053e3kgedgld2ymen8l9yrw8xpjyaal7j5",
            "decimal": 6
        },
        "orai1gzvndtzceqwfymu2kqhta2jn6gmzxvzqwdgvjw": {
            "name": "Milky",
            "denom": "orai1gzvndtzceqwfymu2kqhta2jn6gmzxvzqwdgvjw",
            "decimal": 6,
            "base_denom": "MILKY"
        },
        "ibc/576b1d63e401b6a9a071c78a1d1316d016ec9333d2feb14ad503fac4b8731cd1": {
            "name": "Neutaro",
            "denom": "ibc/576b1d63e401b6a9a071c78a1d1316d016ec9333d2feb14ad503fac4b8731cd1",
            "decimal": 6
        },
        "orai1hn8w33cqvysun2aujk5sv33tku4pgcxhhnsxmvnkfvdxagcx0p8qa4l98q": {
            "name": "OraiDEX",
            "denom": "orai1hn8w33cqvysun2aujk5sv33tku4pgcxhhnsxmvnkfvdxagcx0p8qa4l98q",
            "decimal": 6
        },
        "orai1lplapmgqnelqn253stz6kmvm3ulgdaytn89a8mz9y85xq8wd684s6xl3lt": {
            "name": "Orchai",
            "denom": "orai1lplapmgqnelqn253stz6kmvm3ulgdaytn89a8mz9y85xq8wd684s6xl3lt",
            "decimal": 6
        },
        "orai": {
            "name": "Oraichain",
            "denom": "orai",
            "decimal": 6,
            "base_denom": "orai"
        },
        "orai1lus0f0rhx8s03gdllx2n6vhkmf0536dv57wfge": {
            "name": "OraiDEX",
            "denom": "orai1lus0f0rhx8s03gdllx2n6vhkmf0536dv57wfge",
            "decimal": 6
        },
        "orai19q4qak2g3cj2xc2y3060t0quzn3gfhzx08rjlrdd3vqxhjtat0cq668phq": {
            "name": "Orchai Protocol Staked Compound ATOM",
            "denom": "orai19q4qak2g3cj2xc2y3060t0quzn3gfhzx08rjlrdd3vqxhjtat0cq668phq",
            "decimal": 6
        },
        "ibc/9c4dcd21b48231d0bc2ac3d1b74a864746b37e4292694c93c617324250d002fc": {
            "name": "Osmosis",
            "denom": "ibc/9c4dcd21b48231d0bc2ac3d1b74a864746b37e4292694c93c617324250d002fc",
            "decimal": 6
        },
        "orai1x6uwy6p5rkh4tv44pztafvkj7y9pgqpfyg82006ts4l97fy0kjzs5mzm8p": {
            "name": "Stride Staked Atom",
            "denom": "orai1x6uwy6p5rkh4tv44pztafvkj7y9pgqpfyg82006ts4l97fy0kjzs5mzm8p",
            "decimal": 6
        },
        "orai1d2e02yn3h382ajsqjuz0xlc8qvv5d320m2walmcnfsc406azk9vqf639fa": {
            "name": "Stride Staked Osmo",
            "denom": "orai1d2e02yn3h382ajsqjuz0xlc8qvv5d320m2walmcnfsc406azk9vqf639fa",
            "decimal": 6
        },
        "orai12hzjxfh77wl572gdzct2fxv2arxcwh6gykc7qh": {
            "name": "Tether",
            "denom": "orai12hzjxfh77wl572gdzct2fxv2arxcwh6gykc7qh",
            "decimal": 6
        },
        "orai15un8msx3n5zf9ahlxmfeqd2kwa5wm0nrpxer304m9nd5q6qq0g6sku5pdd": {
            "name": "USDC",
            "denom": "orai15un8msx3n5zf9ahlxmfeqd2kwa5wm0nrpxer304m9nd5q6qq0g6sku5pdd",
            "decimal": 6,
            "base_denom": "USDC"
        },
        "orai1dqa52a7hxxuv8ghe7q5v0s36ra0cthea960q2cukznleqhk0wpnshfegez": {
            "name": "WETH",
            "denom": "orai1dqa52a7hxxuv8ghe7q5v0s36ra0cthea960q2cukznleqhk0wpnshfegez",
            "decimal": 6
        },
        "orai1c7tpjenafvgjtgm9aqwm7afnke6c56hpdms8jc6md40xs3ugd0es5encn0": {
            "name": "Wrapped Tron",
            "denom": "orai1c7tpjenafvgjtgm9aqwm7afnke6c56hpdms8jc6md40xs3ugd0es5encn0",
            "decimal": 6
        },
        "orai1065qe48g7aemju045aeyprflytemx7kecxkf5m7u5h5mphd0qlcs47pclp": {
            "name": "Staking Compound ORAI",
            "denom": "orai1065qe48g7aemju045aeyprflytemx7kecxkf5m7u5h5mphd0qlcs47pclp",
            "decimal": 6
        }
    }
    all = {**orai, **cosmos}


class MulticallContract:
    mapping = {
        "orai": {
            "multicall_balance": "orai1dyljypavg7qpt5d72a48a4pyg38d580aat55qql6tdcwfgydy6jsznk0h5",
            "multicall": "orai1q7x644gmf7h8u8y6y8t9z9nnwl8djkmspypr6mxavsk9ual7dj0sxpmgwd"
        }
    }




import json

GHO_FLASH_MINTER = json.loads("""
[
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "ghoToken",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "ghoTreasury",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "fee",
        "type": "uint256"
      },
      {
        "internalType": "address",
        "name": "addressesProvider",
        "type": "address"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "constructor"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "oldFee",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "newFee",
        "type": "uint256"
      }
    ],
    "name": "FeeUpdated",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "ghoTreasury",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "asset",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "amount",
        "type": "uint256"
      }
    ],
    "name": "FeesDistributedToTreasury",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "receiver",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "initiator",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "address",
        "name": "asset",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "uint256",
        "name": "amount",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "fee",
        "type": "uint256"
      }
    ],
    "name": "FlashMint",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "oldGhoTreasury",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "newGhoTreasury",
        "type": "address"
      }
    ],
    "name": "GhoTreasuryUpdated",
    "type": "event"
  },
  {
    "inputs": [],
    "name": "ADDRESSES_PROVIDER",
    "outputs": [
      {
        "internalType": "contract IPoolAddressesProvider",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "CALLBACK_SUCCESS",
    "outputs": [
      {
        "internalType": "bytes32",
        "name": "",
        "type": "bytes32"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "GHO_TOKEN",
    "outputs": [
      {
        "internalType": "contract IGhoToken",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "MAX_FEE",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "distributeFeesToTreasury",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "token",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "amount",
        "type": "uint256"
      }
    ],
    "name": "flashFee",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "contract IERC3156FlashBorrower",
        "name": "receiver",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "token",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "amount",
        "type": "uint256"
      },
      {
        "internalType": "bytes",
        "name": "data",
        "type": "bytes"
      }
    ],
    "name": "flashLoan",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "getFee",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "getGhoTreasury",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "token",
        "type": "address"
      }
    ],
    "name": "maxFlashLoan",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "newFee",
        "type": "uint256"
      }
    ],
    "name": "updateFee",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "newGhoTreasury",
        "type": "address"
      }
    ],
    "name": "updateGhoTreasury",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  }
]
""")
        


import json

LIDO_ALLOWED_TOKEN_REGISTRY_ABI = json.loads('''
[
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_admin",
        "type": "address"
      },
      {
        "internalType": "address[]",
        "name": "_addTokenToAllowedListRoleHolders",
        "type": "address[]"
      },
      {
        "internalType": "address[]",
        "name": "_removeTokenFromAllowedListRoleHolders",
        "type": "address[]"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "constructor"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "indexed": true,
        "internalType": "bytes32",
        "name": "previousAdminRole",
        "type": "bytes32"
      },
      {
        "indexed": true,
        "internalType": "bytes32",
        "name": "newAdminRole",
        "type": "bytes32"
      }
    ],
    "name": "RoleAdminChanged",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "account",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "sender",
        "type": "address"
      }
    ],
    "name": "RoleGranted",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "account",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "sender",
        "type": "address"
      }
    ],
    "name": "RoleRevoked",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "_token",
        "type": "address"
      }
    ],
    "name": "TokenAdded",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "_token",
        "type": "address"
      }
    ],
    "name": "TokenRemoved",
    "type": "event"
  },
  {
    "inputs": [],
    "name": "ADD_TOKEN_TO_ALLOWED_LIST_ROLE",
    "outputs": [
      {
        "internalType": "bytes32",
        "name": "",
        "type": "bytes32"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "DEFAULT_ADMIN_ROLE",
    "outputs": [
      {
        "internalType": "bytes32",
        "name": "",
        "type": "bytes32"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "REMOVE_TOKEN_FROM_ALLOWED_LIST_ROLE",
    "outputs": [
      {
        "internalType": "bytes32",
        "name": "",
        "type": "bytes32"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_token",
        "type": "address"
      }
    ],
    "name": "addToken",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "name": "allowedTokens",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "decimals",
    "outputs": [
      {
        "internalType": "uint8",
        "name": "",
        "type": "uint8"
      }
    ],
    "stateMutability": "pure",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "getAllowedTokens",
    "outputs": [
      {
        "internalType": "address[]",
        "name": "",
        "type": "address[]"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      }
    ],
    "name": "getRoleAdmin",
    "outputs": [
      {
        "internalType": "bytes32",
        "name": "",
        "type": "bytes32"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "grantRole",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "hasRole",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_token",
        "type": "address"
      }
    ],
    "name": "isTokenAllowed",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "_tokenAmount",
        "type": "uint256"
      },
      {
        "internalType": "address",
        "name": "_token",
        "type": "address"
      }
    ],
    "name": "normalizeAmount",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_token",
        "type": "address"
      }
    ],
    "name": "removeToken",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "renounceRole",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes32",
        "name": "role",
        "type": "bytes32"
      },
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "revokeRole",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "bytes4",
        "name": "interfaceId",
        "type": "bytes4"
      }
    ],
    "name": "supportsInterface",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  }
]
''')



import json

BRAVODELEGATE2 = json.loads("""
[
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "oldAdmin",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "address",
        "name": "newAdmin",
        "type": "address"
      }
    ],
    "name": "NewAdmin",
    "type": "event",
    "signature": "0xf9ffabca9c8276e99321725bcb43fb076a6c66a54b7f21c4e8146d8519b417dc"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "oldImplementation",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "address",
        "name": "newImplementation",
        "type": "address"
      }
    ],
    "name": "NewImplementation",
    "type": "event",
    "signature": "0xd604de94d45953f9138079ec1b82d533cb2160c906d1076d1f7ed54befbca97a"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "oldPendingAdmin",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "address",
        "name": "newPendingAdmin",
        "type": "address"
      }
    ],
    "name": "NewPendingAdmin",
    "type": "event",
    "signature": "0xca4f2f25d0898edd99413412fb94012f9e54ec8142f9b093e7720646a95b16a9"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "id",
        "type": "uint256"
      }
    ],
    "name": "ProposalCanceled",
    "type": "event",
    "signature": "0x789cf55be980739dad1d0699b93b58e806b51c9d96619bfa8fe0a28abaa7b30c"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "id",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "address",
        "name": "proposer",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "address[]",
        "name": "targets",
        "type": "address[]"
      },
      {
        "indexed": false,
        "internalType": "uint256[]",
        "name": "values",
        "type": "uint256[]"
      },
      {
        "indexed": false,
        "internalType": "string[]",
        "name": "signatures",
        "type": "string[]"
      },
      {
        "indexed": false,
        "internalType": "bytes[]",
        "name": "calldatas",
        "type": "bytes[]"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "startBlock",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "endBlock",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "string",
        "name": "description",
        "type": "string"
      }
    ],
    "name": "ProposalCreated",
    "type": "event",
    "signature": "0x7d84a6263ae0d98d3329bd7b46bb4e8d6f98cd35a7adb45c274c8b7fd5ebd5e0"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "id",
        "type": "uint256"
      }
    ],
    "name": "ProposalExecuted",
    "type": "event",
    "signature": "0x712ae1383f79ac853f8d882153778e0260ef8f03b504e2866e0593e04d2b291f"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "id",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "eta",
        "type": "uint256"
      }
    ],
    "name": "ProposalQueued",
    "type": "event",
    "signature": "0x9a2e42fd6722813d69113e7d0079d3d940171428df7373df9c7f7617cfda2892"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "oldProposalThreshold",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "newProposalThreshold",
        "type": "uint256"
      }
    ],
    "name": "ProposalThresholdSet",
    "type": "event",
    "signature": "0xccb45da8d5717e6c4544694297c4ba5cf151d455c9bb0ed4fc7a38411bc05461"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "voter",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "proposalId",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint8",
        "name": "support",
        "type": "uint8"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "votes",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "string",
        "name": "reason",
        "type": "string"
      }
    ],
    "name": "VoteCast",
    "type": "event",
    "signature": "0xb8e138887d0aa13bab447e82de9d5c1777041ecd21ca36ba824ff1e6c07ddda4"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "oldVotingDelay",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "newVotingDelay",
        "type": "uint256"
      }
    ],
    "name": "VotingDelaySet",
    "type": "event",
    "signature": "0xc565b045403dc03c2eea82b81a0465edad9e2e7fc4d97e11421c209da93d7a93"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "oldVotingPeriod",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "newVotingPeriod",
        "type": "uint256"
      }
    ],
    "name": "VotingPeriodSet",
    "type": "event",
    "signature": "0x7e3f7f0708a84de9203036abaa450dccc85ad5ff52f78c170f3edb55cf5e8828"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "account",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "expiration",
        "type": "uint256"
      }
    ],
    "name": "WhitelistAccountExpirationSet",
    "type": "event",
    "signature": "0x4e7b7545bc5744d0e30425959f4687475774b6c7edad77d24cb51c7d967d4515"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "oldGuardian",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "address",
        "name": "newGuardian",
        "type": "address"
      }
    ],
    "name": "WhitelistGuardianSet",
    "type": "event",
    "signature": "0x80a07e73e552148844a9c216d9724212d609cfa54e9c1a2e97203bdd2c4ad341"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "BALLOT_TYPEHASH",
    "outputs": [
      {
        "internalType": "bytes32",
        "name": "",
        "type": "bytes32"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0xdeaaa7cc"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "DOMAIN_TYPEHASH",
    "outputs": [
      {
        "internalType": "bytes32",
        "name": "",
        "type": "bytes32"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x20606b70"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "MAX_PROPOSAL_THRESHOLD",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x25fd935a"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "MAX_VOTING_DELAY",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0xb1126263"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "MAX_VOTING_PERIOD",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0xa64e024a"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "MIN_PROPOSAL_THRESHOLD",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x791f5d23"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "MIN_VOTING_DELAY",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0xe48083fe"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "MIN_VOTING_PERIOD",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x215809ca"
  },
  {
    "constant": false,
    "inputs": [],
    "name": "_acceptAdmin",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0xe9c714f2"
  },
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "address",
        "name": "governorAlpha",
        "type": "address"
      }
    ],
    "name": "_initiate",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0xf9d28b80"
  },
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "address",
        "name": "newPendingAdmin",
        "type": "address"
      }
    ],
    "name": "_setPendingAdmin",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0xb71d1a0c"
  },
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "uint256",
        "name": "newProposalThreshold",
        "type": "uint256"
      }
    ],
    "name": "_setProposalThreshold",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x17ba1b8b"
  },
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "uint256",
        "name": "newVotingDelay",
        "type": "uint256"
      }
    ],
    "name": "_setVotingDelay",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x1dfb1b5a"
  },
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "uint256",
        "name": "newVotingPeriod",
        "type": "uint256"
      }
    ],
    "name": "_setVotingPeriod",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x0ea2d98c"
  },
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "expiration",
        "type": "uint256"
      }
    ],
    "name": "_setWhitelistAccountExpiration",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x4d6733d2"
  },
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "_setWhitelistGuardian",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x99533365"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "admin",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0xf851a440"
  },
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "uint256",
        "name": "proposalId",
        "type": "uint256"
      }
    ],
    "name": "cancel",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x40e58ee5"
  },
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "uint256",
        "name": "proposalId",
        "type": "uint256"
      },
      {
        "internalType": "uint8",
        "name": "support",
        "type": "uint8"
      }
    ],
    "name": "castVote",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x56781388"
  },
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "uint256",
        "name": "proposalId",
        "type": "uint256"
      },
      {
        "internalType": "uint8",
        "name": "support",
        "type": "uint8"
      },
      {
        "internalType": "uint8",
        "name": "v",
        "type": "uint8"
      },
      {
        "internalType": "bytes32",
        "name": "r",
        "type": "bytes32"
      },
      {
        "internalType": "bytes32",
        "name": "s",
        "type": "bytes32"
      }
    ],
    "name": "castVoteBySig",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x3bccf4fd"
  },
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "uint256",
        "name": "proposalId",
        "type": "uint256"
      },
      {
        "internalType": "uint8",
        "name": "support",
        "type": "uint8"
      },
      {
        "internalType": "string",
        "name": "reason",
        "type": "string"
      }
    ],
    "name": "castVoteWithReason",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x7b3c71d3"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "comp",
    "outputs": [
      {
        "internalType": "contract CompInterface",
        "name": "",
        "type": "address"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x109d0af8"
  },
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "uint256",
        "name": "proposalId",
        "type": "uint256"
      }
    ],
    "name": "execute",
    "outputs": [],
    "payable": true,
    "stateMutability": "payable",
    "type": "function",
    "signature": "0xfe0d94c1"
  },
  {
    "constant": true,
    "inputs": [
      {
        "internalType": "uint256",
        "name": "proposalId",
        "type": "uint256"
      }
    ],
    "name": "getActions",
    "outputs": [
      {
        "internalType": "address[]",
        "name": "targets",
        "type": "address[]"
      },
      {
        "internalType": "uint256[]",
        "name": "values",
        "type": "uint256[]"
      },
      {
        "internalType": "string[]",
        "name": "signatures",
        "type": "string[]"
      },
      {
        "internalType": "bytes[]",
        "name": "calldatas",
        "type": "bytes[]"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x328dd982"
  },
  {
    "constant": true,
    "inputs": [
      {
        "internalType": "uint256",
        "name": "proposalId",
        "type": "uint256"
      },
      {
        "internalType": "address",
        "name": "voter",
        "type": "address"
      }
    ],
    "name": "getReceipt",
    "outputs": [
      {
        "components": [
          {
            "internalType": "bool",
            "name": "hasVoted",
            "type": "bool"
          },
          {
            "internalType": "uint8",
            "name": "support",
            "type": "uint8"
          },
          {
            "internalType": "uint96",
            "name": "votes",
            "type": "uint96"
          }
        ],
        "internalType": "struct GovernorBravoDelegateStorageV1.Receipt",
        "name": "",
        "type": "tuple"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0xe23a9a52"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "implementation",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x5c60da1b"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "initialProposalId",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0xfc4eee42"
  },
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "address",
        "name": "timelock_",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "comp_",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "votingPeriod_",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "votingDelay_",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "proposalThreshold_",
        "type": "uint256"
      }
    ],
    "name": "initialize",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0xd13f90b4"
  },
  {
    "constant": true,
    "inputs": [
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "isWhitelisted",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x3af32abf"
  },
  {
    "constant": true,
    "inputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "name": "latestProposalIds",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x17977c61"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "name",
    "outputs": [
      {
        "internalType": "string",
        "name": "",
        "type": "string"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x06fdde03"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "pendingAdmin",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x26782247"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "proposalCount",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0xda35c664"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "proposalMaxOperations",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x7bdbe4d0"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "proposalThreshold",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0xb58131b0"
  },
  {
    "constant": true,
    "inputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "name": "proposals",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "id",
        "type": "uint256"
      },
      {
        "internalType": "address",
        "name": "proposer",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "eta",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "startBlock",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "endBlock",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "forVotes",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "againstVotes",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "abstainVotes",
        "type": "uint256"
      },
      {
        "internalType": "bool",
        "name": "canceled",
        "type": "bool"
      },
      {
        "internalType": "bool",
        "name": "executed",
        "type": "bool"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x013cf08b"
  },
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "address[]",
        "name": "targets",
        "type": "address[]"
      },
      {
        "internalType": "uint256[]",
        "name": "values",
        "type": "uint256[]"
      },
      {
        "internalType": "string[]",
        "name": "signatures",
        "type": "string[]"
      },
      {
        "internalType": "bytes[]",
        "name": "calldatas",
        "type": "bytes[]"
      },
      {
        "internalType": "string",
        "name": "description",
        "type": "string"
      }
    ],
    "name": "propose",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0xda95691a"
  },
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "uint256",
        "name": "proposalId",
        "type": "uint256"
      }
    ],
    "name": "queue",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0xddf0b009"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "quorumVotes",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x24bc1a64"
  },
  {
    "constant": true,
    "inputs": [
      {
        "internalType": "uint256",
        "name": "proposalId",
        "type": "uint256"
      }
    ],
    "name": "state",
    "outputs": [
      {
        "internalType": "enum GovernorBravoDelegateStorageV1.ProposalState",
        "name": "",
        "type": "uint8"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x3e4f49e6"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "timelock",
    "outputs": [
      {
        "internalType": "contract TimelockInterface",
        "name": "",
        "type": "address"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0xd33219b4"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "votingDelay",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x3932abb1"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "votingPeriod",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x02a251a3"
  },
  {
    "constant": true,
    "inputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "name": "whitelistAccountExpirations",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x38bd0dda"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "whitelistGuardian",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0xc5a8425d"
  }
]
""")
        



import json

PERMISSION_MANAGER = json.loads("""
[
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "previousOwner",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "newOwner",
        "type": "address"
      }
    ],
    "name": "OwnershipTransferred",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "user",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "bool",
        "name": "set",
        "type": "bool"
      }
    ],
    "name": "PermissionsAdminSet",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "user",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "uint256",
        "name": "role",
        "type": "uint256"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "whiteLister",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "bool",
        "name": "set",
        "type": "bool"
      }
    ],
    "name": "RoleSet",
    "type": "event"
  },
  {
    "inputs": [],
    "name": "MAX_NUM_OF_ROLES",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address[]",
        "name": "admins",
        "type": "address[]"
      }
    ],
    "name": "addPermissionAdmins",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256[]",
        "name": "roles",
        "type": "uint256[]"
      },
      {
        "internalType": "address[]",
        "name": "users",
        "type": "address[]"
      }
    ],
    "name": "addPermissions",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "user",
        "type": "address"
      }
    ],
    "name": "getUserPermissionAdmin",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "user",
        "type": "address"
      }
    ],
    "name": "getUserPermissions",
    "outputs": [
      {
        "internalType": "uint256[]",
        "name": "",
        "type": "uint256[]"
      },
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "user",
        "type": "address"
      },
      {
        "internalType": "uint256[]",
        "name": "roles",
        "type": "uint256[]"
      }
    ],
    "name": "isInAllRoles",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "user",
        "type": "address"
      },
      {
        "internalType": "uint256[]",
        "name": "roles",
        "type": "uint256[]"
      }
    ],
    "name": "isInAnyRole",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "user",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "role",
        "type": "uint256"
      }
    ],
    "name": "isInRole",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "admin",
        "type": "address"
      }
    ],
    "name": "isPermissionsAdmin",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "user",
        "type": "address"
      }
    ],
    "name": "isUserPermissionAdminValid",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "owner",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address[]",
        "name": "admins",
        "type": "address[]"
      }
    ],
    "name": "removePermissionAdmins",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256[]",
        "name": "roles",
        "type": "uint256[]"
      },
      {
        "internalType": "address[]",
        "name": "users",
        "type": "address[]"
      }
    ],
    "name": "removePermissions",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "renounceOwnership",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "newOwner",
        "type": "address"
      }
    ],
    "name": "transferOwnership",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  }
]
""")
        



import json

PROTOCOLSHARERESERVE = json.loads("""
[
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "address",
        "name": "previousAdmin",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "address",
        "name": "newAdmin",
        "type": "address"
      }
    ],
    "name": "AdminChanged",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "beacon",
        "type": "address"
      }
    ],
    "name": "BeaconUpgraded",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "implementation",
        "type": "address"
      }
    ],
    "name": "Upgraded",
    "type": "event"
  },
  {
    "stateMutability": "payable",
    "type": "fallback"
  },
  {
    "inputs": [],
    "name": "admin",
    "outputs": [
      {
        "internalType": "address",
        "name": "admin_",
        "type": "address"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "newAdmin",
        "type": "address"
      }
    ],
    "name": "changeAdmin",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "implementation",
    "outputs": [
      {
        "internalType": "address",
        "name": "implementation_",
        "type": "address"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "newImplementation",
        "type": "address"
      }
    ],
    "name": "upgradeTo",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "newImplementation",
        "type": "address"
      },
      {
        "internalType": "bytes",
        "name": "data",
        "type": "bytes"
      }
    ],
    "name": "upgradeToAndCall",
    "outputs": [],
    "stateMutability": "payable",
    "type": "function"
  },
  {
    "stateMutability": "payable",
    "type": "receive"
  },
  {
    "inputs": [],
    "name": "ZeroAddressNotAllowed",
    "type": "error"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "comptroller",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "asset",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "amount",
        "type": "uint256"
      }
    ],
    "name": "AssetsReservesUpdated",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "comptroller",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "asset",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "amount",
        "type": "uint256"
      }
    ],
    "name": "FundsReleased",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "uint8",
        "name": "version",
        "type": "uint8"
      }
    ],
    "name": "Initialized",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "previousOwner",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "newOwner",
        "type": "address"
      }
    ],
    "name": "OwnershipTransferStarted",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "previousOwner",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "newOwner",
        "type": "address"
      }
    ],
    "name": "OwnershipTransferred",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "oldPoolRegistry",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "newPoolRegistry",
        "type": "address"
      }
    ],
    "name": "PoolRegistryUpdated",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "token",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "amount",
        "type": "uint256"
      }
    ],
    "name": "SweepToken",
    "type": "event"
  },
  {
    "inputs": [],
    "name": "acceptOwnership",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "name": "assetsReserves",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "comptroller",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "asset",
        "type": "address"
      }
    ],
    "name": "getPoolAssetReserve",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "protocolIncome_",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "riskFund_",
        "type": "address"
      }
    ],
    "name": "initialize",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "owner",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "pendingOwner",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "poolRegistry",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "protocolIncome",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "comptroller",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "asset",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "amount",
        "type": "uint256"
      }
    ],
    "name": "releaseFunds",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "renounceOwnership",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "riskFund",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "poolRegistry_",
        "type": "address"
      }
    ],
    "name": "setPoolRegistry",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_token",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_to",
        "type": "address"
      }
    ],
    "name": "sweepToken",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "newOwner",
        "type": "address"
      }
    ],
    "name": "transferOwnership",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "comptroller",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "asset",
        "type": "address"
      }
    ],
    "name": "updateAssetsState",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_logic",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "admin_",
        "type": "address"
      },
      {
        "internalType": "bytes",
        "name": "_data",
        "type": "bytes"
      }
    ],
    "stateMutability": "payable",
    "type": "constructor"
  }
]
""")
        


import json

WEPIGGY_LENS_ABI_V2 = json.loads('''
[
  {
    "inputs": [
      {
        "internalType": "contract ComptrollerLensInterface",
        "name": "comptroller",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "getAccountLimits",
    "outputs": [
      {
        "components": [
          {
            "internalType": "contract PTokenLensInterface[]",
            "name": "markets",
            "type": "address[]"
          },
          {
            "internalType": "uint256",
            "name": "liquidity",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "shortfall",
            "type": "uint256"
          }
        ],
        "internalType": "struct WePiggyLens.AccountLimits",
        "name": "",
        "type": "tuple"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "contract PTokenLensInterface",
        "name": "pToken",
        "type": "address"
      },
      {
        "internalType": "address payable",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "pTokenBalances",
    "outputs": [
      {
        "components": [
          {
            "internalType": "address",
            "name": "pToken",
            "type": "address"
          },
          {
            "internalType": "uint256",
            "name": "balance",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "borrowBalance",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "exchangeRateMantissa",
            "type": "uint256"
          }
        ],
        "internalType": "struct WePiggyLens.PTokenBalances",
        "name": "",
        "type": "tuple"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "contract PTokenLensInterface[]",
        "name": "pTokens",
        "type": "address[]"
      },
      {
        "internalType": "address payable",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "pTokenBalancesAll",
    "outputs": [
      {
        "components": [
          {
            "internalType": "address",
            "name": "pToken",
            "type": "address"
          },
          {
            "internalType": "uint256",
            "name": "balance",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "borrowBalance",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "exchangeRateMantissa",
            "type": "uint256"
          }
        ],
        "internalType": "struct WePiggyLens.PTokenBalances[]",
        "name": "",
        "type": "tuple[]"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "contract PTokenLensInterface",
        "name": "pToken",
        "type": "address"
      }
    ],
    "name": "pTokenMetadata",
    "outputs": [
      {
        "components": [
          {
            "internalType": "address",
            "name": "pToken",
            "type": "address"
          },
          {
            "internalType": "uint256",
            "name": "exchangeRateCurrent",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "supplyRatePerBlock",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "borrowRatePerBlock",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "reserveFactorMantissa",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "totalBorrows",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "totalReserves",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "totalSupply",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "totalCash",
            "type": "uint256"
          },
          {
            "internalType": "bool",
            "name": "isListed",
            "type": "bool"
          },
          {
            "internalType": "uint256",
            "name": "collateralFactorMantissa",
            "type": "uint256"
          },
          {
            "internalType": "address",
            "name": "underlyingAssetAddress",
            "type": "address"
          },
          {
            "internalType": "uint256",
            "name": "pTokenDecimals",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "underlyingDecimals",
            "type": "uint256"
          }
        ],
        "internalType": "struct WePiggyLens.PTokenMetadata",
        "name": "",
        "type": "tuple"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "contract PTokenLensInterface[]",
        "name": "pTokens",
        "type": "address[]"
      }
    ],
    "name": "pTokenMetadataAll",
    "outputs": [
      {
        "components": [
          {
            "internalType": "address",
            "name": "pToken",
            "type": "address"
          },
          {
            "internalType": "uint256",
            "name": "exchangeRateCurrent",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "supplyRatePerBlock",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "borrowRatePerBlock",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "reserveFactorMantissa",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "totalBorrows",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "totalReserves",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "totalSupply",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "totalCash",
            "type": "uint256"
          },
          {
            "internalType": "bool",
            "name": "isListed",
            "type": "bool"
          },
          {
            "internalType": "uint256",
            "name": "collateralFactorMantissa",
            "type": "uint256"
          },
          {
            "internalType": "address",
            "name": "underlyingAssetAddress",
            "type": "address"
          },
          {
            "internalType": "uint256",
            "name": "pTokenDecimals",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "underlyingDecimals",
            "type": "uint256"
          }
        ],
        "internalType": "struct WePiggyLens.PTokenMetadata[]",
        "name": "",
        "type": "tuple[]"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "contract PiggyBreederLensInterface",
        "name": "piggyBreeder",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_user",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "_pid",
        "type": "uint256"
      }
    ],
    "name": "pendingPiggy",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "contract PiggyBreederLensInterface",
        "name": "piggyBreeder",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_user",
        "type": "address"
      },
      {
        "internalType": "uint256[]",
        "name": "_pids",
        "type": "uint256[]"
      }
    ],
    "name": "pendingPiggyAll",
    "outputs": [
      {
        "internalType": "uint256[]",
        "name": "",
        "type": "uint256[]"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "contract PiggyDistributionLensInterface",
        "name": "piggyDistribution",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      },
      {
        "internalType": "bool",
        "name": "borrowers",
        "type": "bool"
      },
      {
        "internalType": "bool",
        "name": "suppliers",
        "type": "bool"
      }
    ],
    "name": "pendingWpcAccrued",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  }
]
''')